@@ -2,7 +2,7 @@
 // Name:        AudioPropDlg.cpp
 // Author:      Alex Thuering
 // Created:     23.07.2011
-// RCS-ID:      $Id: AudioPropDlg.cpp,v 1.14 2015/12/29 12:14:05 ntalex Exp $
+// RCS-ID:      $Id: AudioPropDlg.cpp,v 1.16 2016/06/04 22:05:36 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -24,21 +24,14 @@
 //*)
 
 //(*IdInit(AudioPropDlg)
-const long AudioPropDlg::ID_STATICTEXT1 = wxNewId();
 const long AudioPropDlg::ID_STATICTEXT2 = wxNewId();
-const long AudioPropDlg::ID_STATICTEXT3 = wxNewId();
 const long AudioPropDlg::ID_STATICTEXT4 = wxNewId();
-const long AudioPropDlg::ID_STATICTEXT5 = wxNewId();
 const long AudioPropDlg::ID_STATICTEXT6 = wxNewId();
-const long AudioPropDlg::ID_STATICTEXT7 = wxNewId();
 const long AudioPropDlg::ID_DST_CHOICE = wxNewId();
 const long AudioPropDlg::ID_A51_CHECK = wxNewId();
-const long AudioPropDlg::ID_STATICTEXT8 = wxNewId();
 const long AudioPropDlg::ID_LANG_CHOICE = wxNewId();
-const long AudioPropDlg::ID_STATICTEXT10 = wxNewId();
 const long AudioPropDlg::ID_TEXTCTRL1 = wxNewId();
 const long AudioPropDlg::ID_SPINBUTTON2 = wxNewId();
-const long AudioPropDlg::ID_STATICTEXT11 = wxNewId();
 const long AudioPropDlg::ID_TEXTCTRL2 = wxNewId();
 const long AudioPropDlg::ID_RADIOBUTTON1 = wxNewId();
 const long AudioPropDlg::ID_RADIOBUTTON2 = wxNewId();
@@ -48,7 +41,7 @@ const long AudioPropDlg::ID_RADIOBUTTON3 = wxNewId();
 const long AudioPropDlg::ID_TEXTCTRL3 = wxNewId();
 const long AudioPropDlg::ID_SPINBUTTON3 = wxNewId();
 const long AudioPropDlg::ID_CALC_GAIN_BT = wxNewId();
-const long AudioPropDlg::ID_STATICTEXT12 = wxNewId();
+const long AudioPropDlg::ID_STATICTEXT_GAIN = wxNewId();
 //*)
 
 BEGIN_EVENT_TABLE(AudioPropDlg,wxDialog)
@@ -87,32 +80,32 @@ AudioPropDlg::AudioPropDlg(wxWindow* parent,Vob* vob, const wxString& audioFile,
 	mainVSizer = new wxBoxSizer(wxVERTICAL);
 	propGridSizer = new wxFlexGridSizer(0, 2, 4, 4);
 	propGridSizer->AddGrowableCol(1);
-	label1 = new wxStaticText(this, ID_STATICTEXT1, _("File Name:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT1"));
+	label1 = new wxStaticText(this, wxID_ANY, _("File Name:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
 	propGridSizer->Add(label1, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
 	m_fileNameText = new wxStaticText(this, ID_STATICTEXT2, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT2"));
-	propGridSizer->Add(m_fileNameText, 1, wxEXPAND|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
-	label2 = new wxStaticText(this, ID_STATICTEXT3, _("Duration:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT3"));
+	propGridSizer->Add(m_fileNameText, 1, wxEXPAND, 5);
+	label2 = new wxStaticText(this, wxID_ANY, _("Duration:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
 	propGridSizer->Add(label2, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
 	m_durText = new wxStaticText(this, ID_STATICTEXT4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT4"));
-	propGridSizer->Add(m_durText, 1, wxEXPAND|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
-	label3 = new wxStaticText(this, ID_STATICTEXT5, _("Source Format:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT5"));
+	propGridSizer->Add(m_durText, 1, wxEXPAND, 5);
+	label3 = new wxStaticText(this, wxID_ANY, _("Source Format:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
 	propGridSizer->Add(label3, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
 	m_srcText = new wxStaticText(this, ID_STATICTEXT6, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT6"));
-	propGridSizer->Add(m_srcText, 1, wxALL|wxEXPAND|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
-	label4 = new wxStaticText(this, ID_STATICTEXT7, _("Destination Format:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT7"));
+	propGridSizer->Add(m_srcText, 1, wxALL|wxEXPAND, 0);
+	label4 = new wxStaticText(this, wxID_ANY, _("Destination Format:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
 	propGridSizer->Add(label4, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
 	boxSizer3 = new wxBoxSizer(wxHORIZONTAL);
 	m_dstChoice = new wxChoice(this, ID_DST_CHOICE, wxDefaultPosition, wxDefaultSize, 0, 0, 0, wxDefaultValidator, _T("ID_DST_CHOICE"));
-	boxSizer3->Add(m_dstChoice, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
+	boxSizer3->Add(m_dstChoice, 0, wxALIGN_CENTER_VERTICAL, 0);
 	m_audio51 = new wxCheckBox(this, ID_A51_CHECK, _("5.1"), wxDefaultPosition, wxSize(-1,-1), 0, wxDefaultValidator, _T("ID_A51_CHECK"));
 	m_audio51->SetValue(false);
-	boxSizer3->Add(m_audio51, 0, wxLEFT|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	boxSizer3->Add(m_audio51, 0, wxLEFT|wxEXPAND, 5);
 	propGridSizer->Add(boxSizer3, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
-	label5 = new wxStaticText(this, ID_STATICTEXT8, _("Language:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT8"));
+	label5 = new wxStaticText(this, wxID_ANY, _("Language:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
 	propGridSizer->Add(label5, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
 	m_langChoice = new wxChoice(this, ID_LANG_CHOICE, wxDefaultPosition, wxDefaultSize, 0, 0, 0, wxDefaultValidator, _T("ID_LANG_CHOICE"));
 	propGridSizer->Add(m_langChoice, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
-	label7 = new wxStaticText(this, ID_STATICTEXT10, _("Time delay:"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT10"));
+	label7 = new wxStaticText(this, wxID_ANY, _("Time delay:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
 	propGridSizer->Add(label7, 1, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 0);
 	boxSizer2 = new wxBoxSizer(wxHORIZONTAL);
 	m_offsetCtrl = new wxTextCtrl(this, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL1"));
@@ -120,50 +113,50 @@ AudioPropDlg::AudioPropDlg(wxWindow* parent,Vob* vob, const wxString& audioFile,
 	m_offsetSpin = new wxSpinButton(this, ID_SPINBUTTON2, wxDefaultPosition, wxDefaultSize, wxSP_VERTICAL|wxSP_ARROW_KEYS, _T("ID_SPINBUTTON2"));
 	m_offsetSpin->SetRange(0, 999);
 	m_offsetSpin->SetMinSize(wxSize(16,12));
-	boxSizer2->Add(m_offsetSpin, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
-	label8 = new wxStaticText(this, ID_STATICTEXT11, _("ms"), wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT11"));
-	boxSizer2->Add(label8, 0, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	boxSizer2->Add(m_offsetSpin, 0, wxEXPAND, 0);
+	label8 = new wxStaticText(this, wxID_ANY, _("ms"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
+	boxSizer2->Add(label8, 0, wxALL|wxEXPAND, 5);
 	propGridSizer->Add(boxSizer2, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
 	label9 = new wxStaticText(this, wxID_ANY, _("Filters:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
 	propGridSizer->Add(label9, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
 	m_filtersCtrl = new wxTextCtrl(this, ID_TEXTCTRL2, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL2"));
-	propGridSizer->Add(m_filtersCtrl, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
-	mainVSizer->Add(propGridSizer, 0, wxTOP|wxLEFT|wxRIGHT|wxEXPAND|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
+	propGridSizer->Add(m_filtersCtrl, 0, wxEXPAND, 5);
+	mainVSizer->Add(propGridSizer, 0, wxTOP|wxLEFT|wxRIGHT|wxEXPAND, 5);
 	volAdjustBox = new wxStaticBoxSizer(wxHORIZONTAL, this, _("Volume adjustment"));
 	GridBagSizer2 = new wxGridBagSizer(4, 4);
 	GridBagSizer2->AddGrowableCol(1);
 	m_volNoBt = new wxRadioButton(this, ID_RADIOBUTTON1, _("No"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_RADIOBUTTON1"));
-	GridBagSizer2->Add(m_volNoBt, wxGBPosition(0, 0), wxDefaultSpan, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	GridBagSizer2->Add(m_volNoBt, wxGBPosition(0, 0), wxDefaultSpan, wxALL|wxEXPAND, 5);
 	m_volChange1Bt = new wxRadioButton(this, ID_RADIOBUTTON2, _("Change (%)"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_RADIOBUTTON2"));
-	GridBagSizer2->Add(m_volChange1Bt, wxGBPosition(1, 0), wxDefaultSpan, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	GridBagSizer2->Add(m_volChange1Bt, wxGBPosition(1, 0), wxDefaultSpan, wxALL|wxEXPAND, 5);
 	m_volume1Ctrl = new wxSpinCtrl(this, ID_SPINCTRL1, _T("100"), wxDefaultPosition, wxSize(60,-1), 0, 0, 999, 100, _T("ID_SPINCTRL1"));
 	m_volume1Ctrl->SetValue(_T("100"));
 	GridBagSizer2->Add(m_volume1Ctrl, wxGBPosition(1, 1), wxDefaultSpan, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
 	m_volNormalizeBt = new wxRadioButton(this, ID_RADIOBUTTON4, _("Normalize"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_RADIOBUTTON4"));
-	GridBagSizer2->Add(m_volNormalizeBt, wxGBPosition(3, 0), wxDefaultSpan, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	GridBagSizer2->Add(m_volNormalizeBt, wxGBPosition(3, 0), wxDefaultSpan, wxALL|wxEXPAND, 5);
 	m_volChange2Bt = new wxRadioButton(this, ID_RADIOBUTTON3, _("Change (dB)"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_RADIOBUTTON3"));
-	GridBagSizer2->Add(m_volChange2Bt, wxGBPosition(2, 0), wxDefaultSpan, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	GridBagSizer2->Add(m_volChange2Bt, wxGBPosition(2, 0), wxDefaultSpan, wxALL|wxEXPAND, 5);
 	BoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
 	m_volume2Ctrl = new wxTextCtrl(this, ID_TEXTCTRL3, wxEmptyString, wxDefaultPosition, wxSize(42,-1), 0, wxDefaultValidator, _T("ID_TEXTCTRL3"));
 	BoxSizer1->Add(m_volume2Ctrl, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
 	m_volume2Spin = new wxSpinButton(this, ID_SPINBUTTON3, wxDefaultPosition, wxDefaultSize, wxSP_VERTICAL|wxSP_ARROW_KEYS, _T("ID_SPINBUTTON3"));
 	m_volume2Spin->SetRange(-999999, 999999);
 	m_volume2Spin->SetMinSize(wxSize(16,12));
-	BoxSizer1->Add(m_volume2Spin, 0, wxLEFT|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 2);
-	GridBagSizer2->Add(BoxSizer1, wxGBPosition(2, 1), wxDefaultSpan, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	BoxSizer1->Add(m_volume2Spin, 0, wxLEFT|wxEXPAND, 2);
+	GridBagSizer2->Add(BoxSizer1, wxGBPosition(2, 1), wxDefaultSpan, wxEXPAND, 5);
 	normBoxSizer = new wxBoxSizer(wxHORIZONTAL);
 	m_calcGainBt = new wxBitmapButton(this, ID_CALC_GAIN_BT, wxBITMAP_FROM_MEMORY(refresh), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("ID_CALC_GAIN_BT"));
 	normBoxSizer->Add(m_calcGainBt, 0, wxRIGHT|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
-	m_gainText = new wxStaticText(this, ID_STATICTEXT12, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT12"));
-	normBoxSizer->Add(m_gainText, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
-	GridBagSizer2->Add(normBoxSizer, wxGBPosition(3, 1), wxDefaultSpan, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	m_gainText = new wxStaticText(this, ID_STATICTEXT_GAIN, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, _T("ID_STATICTEXT_GAIN"));
+	normBoxSizer->Add(m_gainText, 1, wxALIGN_CENTER_VERTICAL, 5);
+	GridBagSizer2->Add(normBoxSizer, wxGBPosition(3, 1), wxDefaultSpan, wxEXPAND, 5);
 	volAdjustBox->Add(GridBagSizer2, 1, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
-	mainVSizer->Add(volAdjustBox, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	mainVSizer->Add(volAdjustBox, 1, wxALL|wxEXPAND, 5);
 	stdDialogButtonSizer = new wxStdDialogButtonSizer();
 	stdDialogButtonSizer->AddButton(new wxButton(this, wxID_OK, wxEmptyString));
 	stdDialogButtonSizer->AddButton(new wxButton(this, wxID_CANCEL, wxEmptyString));
 	stdDialogButtonSizer->Realize();
-	mainVSizer->Add(stdDialogButtonSizer, 0, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
+	mainVSizer->Add(stdDialogButtonSizer, 0, wxALL|wxEXPAND, 5);
 	SetSizer(mainVSizer);
 	mainVSizer->Fit(this);
 	mainVSizer->SetSizeHints(this);
@@ -205,10 +198,20 @@ AudioPropDlg::AudioPropDlg(wxWindow* parent,Vob* vob, const wxString& audioFile,
 	m_durText->SetLabel(s);
 	m_srcText->SetLabel(m_stream->GetSourceFormat());
 
-	m_dstChoice->Append(DVD::GetAudioFormatLabels(true, true));
-	m_dstChoice->SetSelection(m_stream->GetAudioFormat());
+	bool copyEnabled = m_stream->IsCopyPossible();
+	int afIdx = m_stream->GetAudioFormat();
+	if (!copyEnabled && afIdx > 0)
+		afIdx -= 1;
+	m_dstChoice->Append(DVD::GetAudioFormatLabels(copyEnabled, true));
+	m_dstChoice->SetSelection(afIdx);
 
-	m_audio51->SetValue(m_stream->GetChannelNumber() == 6);
+	// upmix to 5.1 oder donwmix to stereo
+	if (m_stream->GetSourceChannelNumber() >= 6) {
+		m_audio51->SetLabel(_("stereo"));
+		m_audio51->SetValue(m_stream->GetChannelNumber() == 2);
+	} else {
+		m_audio51->SetValue(m_stream->GetChannelNumber() == 6);
+	}
 
 	m_langChoice->Append(DVD::GetLanguageNames());
 	map<wxString, wxString>& langMap = DVD::GetLanguageMap();
@@ -273,9 +276,12 @@ AudioPropDlg::~AudioPropDlg() {
 }
 
 void AudioPropDlg::OnChangeFormat(wxCommandEvent& event) {
-	m_audio51->Enable(GetAudioFormat() == afAC3 && m_stream->GetSourceChannelNumber() != 6);
 	if (!m_audio51->IsEnabled())
-		m_audio51->SetValue(m_stream->GetSourceChannelNumber() == 6);
+		m_audio51->SetValue(false);
+	m_audio51->Enable(GetAudioFormat() >= afAC3);
+	if (!m_audio51->IsEnabled()) {
+		m_audio51->SetValue(GetAudioFormat() == afMP2 && m_stream->GetSourceChannelNumber() > 2);
+	}
 	bool allowChangeVolume = GetAudioFormat() != afNONE && GetAudioFormat() != afCOPY;
 	m_volNoBt->Enable(allowChangeVolume);
 	m_volChange1Bt->Enable(allowChangeVolume);
@@ -289,7 +295,9 @@ void AudioPropDlg::OnChangeFormat(wxCommandEvent& event) {
 }
 
 AudioFormat AudioPropDlg::GetAudioFormat() {
-	return (AudioFormat) m_dstChoice->GetSelection();
+	bool copyEnabled = m_stream->IsCopyPossible();
+	int afIdx = m_dstChoice->GetSelection();
+	return (AudioFormat) (copyEnabled || afIdx == 0 ? afIdx : afIdx + 1);
 }
 
 wxString AudioPropDlg::GetLangCode() {
@@ -338,6 +346,7 @@ void AudioPropDlg::OnSelectVolumeBt(wxCommandEvent& event){
 	m_volume1Ctrl->Enable(m_volChange1Bt->GetValue());
 	m_volume2Ctrl->Enable(m_volChange2Bt->GetValue());
 	m_volume2Spin->Enable(m_volChange2Bt->GetValue());
+	m_calcGainBt->Enable(m_volNormalizeBt->GetValue());
 }
 
 void AudioPropDlg::OnOk(wxCommandEvent& event) {
@@ -352,7 +361,9 @@ void AudioPropDlg::OnOk(wxCommandEvent& event) {
 			return;
 		}
 	}
-	m_stream->SetChannelNumber(m_audio51->IsEnabled() && m_audio51->GetValue() ? 6 : -1);
+	// upmix to 5.1 oder donwmix to stereo
+	m_stream->SetChannelNumber(m_audio51->IsEnabled() && m_audio51->GetValue() ?
+			(m_stream->GetSourceChannelNumber() >= 6 ? 2 : 6) : -1);
 	if (m_volNoBt->GetValue()) {
 		m_stream->SetAudioVolume(1.0);
 		m_stream->SetAudioAdjustType(aatChangePercent);
@@ -386,17 +397,35 @@ void AudioPropDlg::OnCalculateGain(wxCommandEvent& event) {
 	pDlg.Show();
 	pDlg.SetFocus();
 	pDlg.Pulse();
+	
 	AVConvTimeProgExecute prog(&pDlg);
-	if (prog.Execute(transcoder.GetCmd()) && prog.IsOk()) {
+	wxString cmd = transcoder.GetCmd(true);
+	if (!prog.Execute(cmd) || !prog.IsOk()) {
 		pDlg.Hide();
-		m_stream->SetTrackGain(prog.GetTrackGain());
-		m_stream->SetReplayGainCalculated(true);
-		m_gainText->SetLabel(_("Track gain:") + wxString::Format(wxT(" %+0.1f dB"), m_stream->GetTrackGain()));
-	} else {
-		pDlg.Hide();
-#if wxCHECK_VERSION(2,9,0)
 		if (!pDlg.WasCancelled())
 			wxLogError(wxT("Failed calculation of replay gain"));
-#endif
+		return;
 	}
+	
+	double trackGain = prog.GetTrackGain();
+	if (trackGain > 0) {
+		wxFfmpegMediaTranscoder transcoder2;
+		transcoder2.AddInputFile(m_audioFile.length() ? m_audioFile : m_vob->GetFilename());
+		transcoder2.VolumeDetect(audioStreamIdx);
+		AVConvTimeProgExecute prog2(&pDlg);
+		if (!prog2.Execute(transcoder2.GetCmd(true)) || !prog2.IsOk()) {
+			pDlg.Hide();
+			if (!pDlg.WasCancelled())
+				wxLogError(wxT("Failed calculation of replay gain"));
+			return;
+		}
+		if (trackGain > -prog2.GetMaxVolume())
+			trackGain = -prog2.GetMaxVolume();
+	}
+	
+	pDlg.Hide();
+	m_stream->SetTrackGain(trackGain);
+	m_stream->SetReplayGainCalculated(true);
+	m_gainText->SetLabel(_("Track gain:") + wxString::Format(wxT(" %+0.1f dB"), m_stream->GetTrackGain()));
+	Fit();
 }
