@@ -3,7 +3,7 @@
 // Purpose:     Configuration
 // Author:      Alex Thuering
 // Created:     27.03.2003
-// RCS-ID:      $Id: Config.h,v 1.101 2015/08/02 17:30:28 ntalex Exp $
+// RCS-ID:      $Id: Config.h,v 1.110 2017/11/25 14:39:34 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -21,6 +21,7 @@ const wxString DEF_VERSION		= wxT("");
 const int DEF_LANGUAGE			= wxLANGUAGE_UNKNOWN;
 const wxString DEF_LANGUAGE_CODE= wxT("");
 const wxString DEF_DISC_LABEL	= wxT("DVD");
+const int DEF_DVD_RESOLUTION	= 0;
 const int DEF_DISC_CAPACITY		= 3;
 const int DEF_POST_COMMAND		= 0;
 const int DEF_VIDEO_FORMAT		= 2;
@@ -31,8 +32,8 @@ const int DEF_ASPECT_RATIO		= 1;
 const int DEF_CHAPTER_LENGTH	= 10;
 const wxString DEF_BUTTON		= wxT("frame-v2.xml");
 const wxString DEF_TEXT_BUTTON	= wxT("text-v3.xml");
-const int DEF_SLIDE_DURATION = 5;
-const wxString DEF_TRANSITION = wxT("fade.xml");
+const int DEF_SLIDE_DURATION	= 5;
+const wxString DEF_TRANSITION	= wxT("fade.xml");
 const double DEF_TRANSITION_DURATION = 1.0;
 const wxString DEF_FILE_BROWSER_DIR = wxT("");
 #define DEF_FILE_BROWSER_LAST_DIR wxGetHomeDir()
@@ -42,8 +43,8 @@ const int DEF_UNDO_HISTORY_SIZE = 100;
 const int DEF_VIDEO_BITRATE = -1;
 const int DEF_AUDIO_BITRATE = 192;
 const int DEF_MENU_FRAME_COUNT = 12;
-const int DEF_MENU_VIDEO_BITRATE = 6000;
-const int DEF_SLIDESHOW_VIDEO_BITRATE = 6000;
+const int DEF_MENU_VIDEO_BITRATE = 8000;
+const int DEF_SLIDESHOW_VIDEO_BITRATE = 8000;
 const int DEF_THREAD_COUNT = wxThread::GetCPUCount() > 0 ? wxThread::GetCPUCount() : 1;
 const int DEF_DVD_RESERVED_SPACE = 0; // KB
 #define DEF_GEN_TEMP_DIR wxFileName::GetTempDir()
@@ -68,10 +69,17 @@ const bool DEF_USE_MPLEX = false;
 const bool DEF_USE_MPLEX_FOR_MENUS = true;
 #ifdef USE_FFMPEG
 const wxString DEF_AVCONV_CMD    = _T("ffmpeg");
+const wxString DEF_PLAY_CMD    = _T("ffplay");
 #else
 const wxString DEF_AVCONV_CMD    = _T("avconv");
+const wxString DEF_PLAY_CMD    = _T("avplay");
 #endif
-const wxString DEF_MPLEX_CMD     = _T("mplex -f 8 -S 0 -M -V -o \"$FILE_OUT\" \"$FILE_VIDEO\" \"$FILE_AUDIO\"");
+#ifdef __WXMSW__
+	const wxString DEF_ENCODER   = _T("ffmpeg-vbr");
+#else
+	const wxString DEF_ENCODER   = _T("");
+#endif
+const wxString DEF_MPLEX_CMD     = _T("mplex -f 8 -S 0 -M -V $PARAMS -o \"$FILE_OUT\" \"$FILE_VIDEO\" \"$FILE_AUDIO\"");
 const wxString DEF_SPUMUX_CMD    = _T("spumux -P -s $STREAM \"$FILE_CONF\"");
 const wxString DEF_DVDAUTHOR_CMD = _T("dvdauthor -o \"$DIR\" -x \"$FILE_CONF\"");
 const wxString DEF_ISO_CMD       = _T("mkisofs -V \"$VOL_ID\" -o \"$FILE\" -dvd-video \"$DIR\"");
@@ -143,6 +151,7 @@ public:
 	CONFIG_PROP(Version,  _T("Version"), DEF_VERSION)
 	CONFIG_PROP(LanguageCode, _T("Interface/LanguageCode"), DEF_LANGUAGE_CODE)
     CONFIG_PROP(DefDiscLabel, _T("Interface/DefDiscLabel"), DEF_DISC_LABEL)
+	CONFIG_PROP_INT(DefDvdResolution, _T("Interface/DefDvdResolution"), DEF_DVD_RESOLUTION)
     CONFIG_PROP_INT(DefDiscCapacity, _T("Interface/DefDiscCapacity"), DEF_DISC_CAPACITY)
     CONFIG_PROP_INT(DefPostCommand, _T("Interface/DefPostCommand"), DEF_POST_COMMAND)
 	
@@ -156,6 +165,7 @@ public:
 	CONFIG_PROP_BOOL(DefAudio51, _T("Interface/DefAudio51"), false)
 	CONFIG_PROP_BOOL(DefAudioNormalize, _T("Interface/DefAudioNormalize"), false)
 	CONFIG_PROP_INT(DefChapterLength, _T("Interface/DefChapterLength"), DEF_CHAPTER_LENGTH)
+	CONFIG_PROP_BOOL(AddChapterAtTitleEnd, _T("Interface/AddChapterAtTitleEnd"), false)
 	CONFIG_PROP(DefButton, _T("Interface/DefButton"), DEF_BUTTON)
 	CONFIG_PROP(DefTextButton, _T("Interface/DefTextButton"), DEF_TEXT_BUTTON)
 	CONFIG_PROP_BOOL(DefPlayAllTitlesets, _T("Interface/DefPlayAllTitlesets"), false)
@@ -202,13 +212,21 @@ public:
 	CONFIG_PROP_INT(AudioBitrate, _T("Generate/AudioBitrate"), DEF_AUDIO_BITRATE)
 	CONFIG_PROP_INT(MenuFrameCount,_T("Generate/MenuFrameCount"), DEF_MENU_FRAME_COUNT)
 	CONFIG_PROP_INT(MenuVideoBitrate, _T("Generate/MenuVideoBitrate"), DEF_MENU_VIDEO_BITRATE)
+	CONFIG_PROP_BOOL(MenuVideoCBR, _T("Generate/MenuVideoCBR"), false)
+	CONFIG_PROP_BOOL(AllowHdMenues, _T("Generate/AllowHdMenues"), false)
+	CONFIG_PROP_BOOL(AllowHdTitles, _T("Generate/AllowHdTitles"), false)
 	CONFIG_PROP_BOOL(DrawButtonsOnBackground, _T("Generate/DrawButtonsOnBackground"), true)
+	CONFIG_PROP_BOOL(ButtonsOffset2px, _T("Generate/ButtonsOffset2px"), false)
     CONFIG_PROP_INT(SlideshowVideoBitrate, _T("Generate/SlideshowVideoBitrate"), DEF_SLIDESHOW_VIDEO_BITRATE)
+	CONFIG_PROP_BOOL(SlideshowVideoCBR, _T("Generate/SlideshowVideoCBR"), false)
     CONFIG_PROP_INT(ThreadCount, _T("Generate/ThreadCount"), DEF_THREAD_COUNT)
     CONFIG_PROP_INT(DvdReservedSpace, _T("Generate/DvdReservedSpace"), DEF_DVD_RESERVED_SPACE)
     CONFIG_PROP_BOOL(UseMplex, _T("Generate/UseMplex"), DEF_USE_MPLEX)
     CONFIG_PROP_BOOL(UseMplexForMenus, _T("Generate/UseMplexForMenus"), DEF_USE_MPLEX_FOR_MENUS)
+    CONFIG_PROP(PlayCmd, _T("Generate/PlayCmd"), DEF_PLAY_CMD)
     CONFIG_PROP(AVConvCmd, _T("Generate/AVConvCmd"), DEF_AVCONV_CMD)
+	CONFIG_PROP(Encoder, _T("Generate/Encoder"), DEF_ENCODER)
+    CONFIG_PROP(EncoderMode, _T("Generate/EncoderMode"), wxT(""))
     CONFIG_PROP(MplexCmd, _T("Generate/MplexCmd"), DEF_MPLEX_CMD)
 	CONFIG_PROP(SpumuxCmd, _T("Generate/SpumuxCmd"), DEF_SPUMUX_CMD)
 	CONFIG_PROP(DvdauthorCmd, _T("Generate/DvdauthorCmd"), DEF_DVDAUTHOR_CMD)
