@@ -3,11 +3,17 @@
 // Purpose:     The menu cells dialog
 // Author:      Alex Thuering
 // Created:     10.01.2012
-// RCS-ID:      $Id: MenuCellsDlg.cpp,v 1.6 2015/10/03 13:29:57 ntalex Exp $
+// RCS-ID:      $Id: MenuCellsDlg.cpp,v 1.7 2016/01/06 21:10:04 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 #include "MenuCellsDlg.h"
+#include "MenuPropDlg.h"
+#include "Utils.h"
+#include <wxVillaLib/utils.h>
+#include <algorithm>
+#include "rc/add.png.h"
+#include "rc/remove.png.h"
 
 //(*InternalHeaders(MenuCellsDlg)
 #include <wx/intl.h>
@@ -15,13 +21,6 @@
 #include <wx/string.h>
 //*)
 
-#include "TitlePropDlg.h"
-#include "Utils.h"
-#include <wxVillaLib/utils.h>
-#include <algorithm>
-#include "rc/add.png.h"
-#include "rc/remove.png.h"
-
 class CellsTable: public wxGridTableBase {
 public:
 	CellsTable(vector<Cell*>& cells): wxGridTableBase(), m_cells(cells) { }
@@ -127,7 +126,8 @@ BEGIN_EVENT_TABLE(MenuCellsDlg ,wxDialog)
 	EVT_BUTTON(wxID_OK, MenuCellsDlg::OnOkBt)
 END_EVENT_TABLE()
 
-MenuCellsDlg::MenuCellsDlg(wxWindow* parent, DVD* dvd, int tsi, vector<Cell*>& cells): m_dvd(dvd), m_tsi(tsi) {
+MenuCellsDlg::MenuCellsDlg(MenuPropDlg* parent, DVD* dvd, int tsi, int pgci, vector<Cell*>& cells):
+		parentDlg(parent), m_dvd(dvd), m_tsi(tsi), m_pgci(pgci) {
 	VECTOR_COPY(cells, m_cells, Cell)
 
 	//(*Initialize(MenuCellsDlg)
@@ -195,7 +195,7 @@ void MenuCellsDlg::SetCellEditors(int row) {
 	const wxString cellTypes[] = { _("Regular"), _("Program"), _("Chapter") };
 	m_grid->SetCellEditor(row, 0, new wxGridCellChoiceEditor(WXSIZEOF(cellTypes), cellTypes));
 	m_grid->SetCellEditor(row, 3, new wxGridCellNumberEditor(-1, 255));
-	m_grid->SetCellEditor(row, 4, new wxGridCellChoiceEditor(TitlePropDlg::GetCommandList(m_dvd, m_tsi), true));
+	m_grid->SetCellEditor(row, 4, new wxGridCellChoiceEditor(parentDlg->GetCommandList(), true));
 }
 
 void MenuCellsDlg::OnAddBt(wxCommandEvent& event) {
@@ -221,6 +221,14 @@ void MenuCellsDlg::OnDelBtClick(wxCommandEvent& event) {
 bool cellCompare(Cell* c1, Cell* c2) { return c1->GetStart() < c2->GetStart(); }
 
 void MenuCellsDlg::OnOkBt(wxCommandEvent& event) {
+	// check commands
+	for (unsigned int i = 0; i < m_cells.size(); i++) {
+		DVDAction action;
+		action.SetCustom(m_cells[i]->GetCommands());
+		if (m_cells[i]->GetCommands().length() && !action.IsValid(m_dvd, m_tsi, m_pgci, true, wxT(""), true, false)) {
+			return;
+		}
+	}
 	// sort cells
 	std::sort(m_cells.begin(), m_cells.end(), cellCompare);
 	m_grid->Refresh();
