@@ -3,7 +3,7 @@
 // Purpose:     FFMPEG Media Transcoder
 // Author:      Alex Thuering
 // Created:     26.04.2008
-// RCS-ID:      $Id: mediatrc_ffmpeg.cpp,v 1.73 2015/09/25 13:33:16 ntalex Exp $
+// RCS-ID:      $Id: mediatrc_ffmpeg.cpp,v 1.80 2017/11/25 14:40:13 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -11,6 +11,7 @@
 #include "mediatrc_ffmpeg.h"
 #include "Utils.h"
 #include "Config.h"
+#include <wxVillaLib/utils.h>
 #include <wx/wx.h>
 
 wxFfmpegMediaTranscoder::wxFfmpegMediaTranscoder() {
@@ -29,8 +30,19 @@ void wxFfmpegMediaTranscoder::Init() {
 }
 
 /** Returns transcoding command */
-wxString wxFfmpegMediaTranscoder::GetCmd() const {
-	return s_config.GetAVConvCmd() + m_cmd;
+wxString wxFfmpegMediaTranscoder::GetCmd(bool forceInternalEncoder) const {
+	wxString cmd = s_config.GetAVConvCmd();
+	if (s_config.GetEncoder().length() && !forceInternalEncoder) {
+		cmd = s_config.GetEncoder();
+#ifdef __WXMSW__
+		if (wxFileExists(wxGetAppPath() + cmd + wxT(".bat")))
+			cmd += wxT(".bat");
+#endif
+		if (s_config.GetEncoderMode().length() > 1) {
+			cmd += wxT(" -mode " + s_config.GetEncoderMode());
+		}
+	}
+	return cmd + m_cmd;
 }
 
 void wxFfmpegMediaTranscoder::AddOption(const wxString& name, const wxString& value) {
@@ -51,12 +63,17 @@ void wxFfmpegMediaTranscoder::AddSubtitleOption(const wxString& name, int stream
 
 bool wxFfmpegMediaTranscoder::AddInputFile(const wxString& fileName, const wxString& format, long tsOffset) {
 	if (tsOffset > 0)
-		AddOption(wxT("itsoffset"), Time2String(tsOffset));
-	if (fileName == wxT("/dev/zero")) {
-		m_cmd += wxT(" -ac 2 -ar 48000 -f s16le -i /dev/zero");
+		AddOption("itsoffset", Time2String(tsOffset));
+	if (fileName == "/dev/zero") {
+		m_cmd += " -ac 2 -ar 48000 -f s16le -i /dev/zero";
 	} else {
-		if (format.length())
-			AddOption(wxT("f"), format);
+		if (format.length()) {
+			if ((int) format.Index(':') > 0) {
+				AddOption(wxT("f"), format.BeforeFirst(':'));
+				AddOption(wxT("ac"), format.AfterFirst(':'));
+			} else
+				AddOption(wxT("f"), format);
+		}
 		m_cmd += wxString::Format(wxT(" -i \"%s\""), fileName.c_str());
 	}
 	
@@ -66,7 +83,7 @@ bool wxFfmpegMediaTranscoder::AddInputFile(const wxString& fileName, const wxStr
 
 bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName, VideoFormat videoFormat, bool ntscFilm,
 		AspectRatio aspectRatio, AudioFormat audioFormat, SubtitleFormat subtitleFormat, int videoBitrate,
-		bool vbr, int audioBitrate, int mapFileIdx, int mapStreamIdx) {
+		bool vbr, int audioBitrate, int mapFileIdx, int mapStreamIdx, StreamType mapStreamType) {
 	wxArrayInt audioFormats;
 	if (audioFormat != afNONE)
 		audioFormats.Add(audioFormat);
@@ -74,12 +91,12 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName, VideoForma
 	if (subtitleFormat != sfNONE)
 		subtitleFormats.Add(subtitleFormat);
 	return SetOutputFile(fileName, videoFormat, ntscFilm, aspectRatio, audioFormats, subtitleFormats, videoBitrate,
-			vbr, audioBitrate, mapFileIdx, mapStreamIdx);
+			vbr, audioBitrate, mapFileIdx, mapStreamIdx, mapStreamType);
 }
 
 bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName, VideoFormat videoFormat, bool ntscFilm,
 		AspectRatio aspectRatio, wxArrayInt audioFormats, wxArrayInt subtitleFormats, int videoBitrate, bool vbr,
-		int audioBitrate, int mapFileIdx, int mapStreamIdx) {
+		int audioBitrate, int mapFileIdx, int mapStreamIdx, StreamType mapStreamType) {
 	if (m_outputFormat.length() > 0) {
 		AddOption(wxT("f"), m_outputFormat);
 	} else if (audioFormats.size() == 0) {
@@ -94,6 +111,7 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName, VideoForma
 	
 	if (videoFormat == vfCOPY) {
 		AddVideoOption(wxT("c"), 0, wxT("copy"));
+		AddVideoOption(wxT("bufsize"), 0, wxT("1835008")); // 224*1024*8;
 	} else if (videoFormat != vfNONE) {
 		AddVideoOption(wxT("c"), 0, wxT("mpeg2video"));
 		if (aspectRatio != arAUTO) {
@@ -110,6 +128,7 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName, VideoForma
 			AddOption(wxT("r"), ntscFilm ? wxT("24000/1001") : wxT("30000/1001"));
 			AddOption(wxT("g"), wxT("18"));
 		}
+		AddOption(wxT("pix_fmt"), wxT("yuv420p")); 
 
 		AddVideoOption(wxT("b"), 0, wxString::Format(wxT("%d"), videoBitrate * 1000));
 		AddVideoOption(wxT("maxrate"), 0, wxString::Format(wxT("%d"), vbr ? 9000000 : videoBitrate * 1000));
@@ -130,7 +149,7 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName, VideoForma
 	
 	if (audioFormats.size() > 0) {
 		AddOption(wxT("b:a"), wxString::Format(wxT("%d"), audioBitrate * 1000));
-		AddOption(wxT("ar"), wxT("48000"));
+		AddOption(wxT("ar"), wxT("48000")); // audio sample rate
 	}
 	
 	int outputStreamIdx = 0;
@@ -153,15 +172,22 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName, VideoForma
 	}
 	
 	if (mapFileIdx >= 0) {
-		if (mapStreamIdx >= 0)
-			AddOption(wxT("map"), wxString::Format(wxT("%d:%d"), mapFileIdx, mapStreamIdx));
-		else
+		if (mapStreamIdx >= 0) {
+			if (mapStreamType != stUNKNOWN) {
+				wxChar stTypeChar[] = { wxT('v'), wxT('a'), wxT('s') };
+				AddOption(wxT("map"), wxString::Format(wxT("%d:%c:%d"), mapFileIdx,
+						stTypeChar[mapStreamType - 1], mapStreamIdx));
+			} else {
+				AddOption(wxT("map"), wxString::Format(wxT("%d:%d"), mapFileIdx, mapStreamIdx));
+			}
+		} else {
 			AddOption(wxT("map"), wxString::Format(wxT("%d"), mapFileIdx));
+		}
 	} else {
 		// map all input files into one
 		for (unsigned int i = 0; i < m_inputFileCount; i++) {
 			if (videoFormat != vfNONE && i == 0) // only first file can contain video stream
-				AddOption(wxT("map"), wxString::Format(wxT("%d:v"), i));
+				AddOption(wxT("map"), wxString::Format(wxT("%d:V"), i));
 			if (i > 0 || m_inputFileCount < 1 + audioFormats.size()) // first file can contain no audio streams
 				AddOption(wxT("map"), wxString::Format(wxT("%d:a"), i));
 		}
@@ -192,8 +218,10 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName, VideoForma
 
 /** Sets interlaced encoding flag */
 void wxFfmpegMediaTranscoder::SetInterlaced(bool value) {
-	if (value)
+	if (value) {
 		AddVideoOption(wxT("flags"), 0, wxT("+ilme+ildct"));
+		AddVideoOption(wxT("alternate_scan"), 0, wxT("1"));
+	}
 }
 
 /** Sets field first flag (Auto, TFF, BFF) */
@@ -268,3 +296,16 @@ void wxFfmpegMediaTranscoder::ReplayGain(int audioStreamIdx) {
 	m_cmd += wxT(" /dev/null");
 #endif
 }
+
+/** Set filter and output to calculate volume peak value */
+void wxFfmpegMediaTranscoder::VolumeDetect(int audioStreamIdx) {
+	AddOption(wxT("f"), wxT("null"));
+	AddOption(wxT("map"), wxString::Format(wxT("0:a:%d"), audioStreamIdx));
+	AddOption(wxT("map"), wxT("-0:v"));
+	AddAudioOption(wxT("filter"), 0, wxT("\"volumedetect\""));
+#ifdef __WXMSW__
+	m_cmd += wxT(" NUL");
+#else
+	m_cmd += wxT(" /dev/null");
+#endif
+}
